/* $NetBSD$ */

/*
 * File "hashtable.h" is part of the UDFclient toolkit and part of Artdraw.
 * File $Id: hashtable.h,v 1.6 2003/09/14 14:18:59 imago Exp $ $Name: release-ro1 $
 *
 * Copyright (c) 2003 Reinoud Zandijk <reinoud@netbsd.org>
 * All rights reserved.
 *
 * The UDFclient toolkit is distributed under the Clarified Artistic Licence.
 * A copy of the licence is included in the distribution as
 * `LICENCE.clearified.artistic' and a copy of the licence can also be
 * requested at the GNU foundantion's website.
 *
 * Visit the UDFclient toolkit homepage http://www.13thmonkey.org/udfclient/
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#ifndef _HASHTABLE_H_
#define _HASHTABLE_H_

#include <sys/types.h>
#ifndef NO_STDINT
#	include <stdint.h>
#endif

#include <stdio.h>
#include <stdlib.h>


typedef int (*hash_compare)(void *v1, void *v2);
typedef uint32_t (*hash_calc_hash)(void *v1);


struct hash_entry {
	uint32_t hash_value;
	void	*key;
	void    *entry;			/* generic entry for pointers and/or ints */
	struct   hash_entry *next;
	struct   hash_entry *seq_next;
	struct	 hash_entry *seq_prev;
};


struct hash_table {
	int	bucket_mask;
	int	num_entries;

	hash_compare   compare;
	hash_calc_hash calc;

	struct hash_entry *seq_head;
	struct hash_entry **entries;
};


extern int hash_init_generic_table(struct hash_table *table, int num_buckets, hash_compare compare, hash_calc_hash calc);
extern int hash_init_name_table(struct hash_table *table, int num_buckets);
extern int hash_init_numeric_table(struct hash_table *table, int num_buckets);
extern int hash_add_entry(struct hash_table *table, void *key, void *entry);
extern int hash_delete_entry(struct hash_table *table, void *key);
extern int hash_delete_entry_by_record(struct hash_table *table, struct hash_entry *entry);
extern int hash_lookup_entry(struct hash_table *table, void *key, void **entry);

extern int hash_delete_all(struct hash_table *table);


#endif /* _HASHTABLE_H_ */

