/* $NetBSD$ */

/*
 * File "udf.h" is part of the UDFclient toolkit.
 * File $Id: udf.h,v 1.37 2003/09/14 14:19:01 imago Exp $ $Name: release-ro1 $
 *
 * Copyright (c) 2003 Reinoud Zandijk <reinoud@netbsd.org>
 * All rights reserved.
 *
 * The UDFclient toolkit is distributed under the Clarified Artistic Licence.
 * A copy of the licence is included in the distribution as
 * `LICENCE.clearified.artistic' and a copy of the licence can also be
 * requested at the GNU foundantion's website.
 *
 * Visit the UDFclient toolkit homepage http://www.13thmonkey.org/udfclient/
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#ifndef _UDF_H_
#define _UDF_H_

#include <dirent.h>
#include "osta.h"
#include "ecma167-udf.h"
#include "hashtable.h"


/* exported flags */
extern int udf_verbose;

#define UDF_VERBLEV_NONE	   0
#define UDF_VERBLEV_ACTIONS	   1
#define UDF_VERBLEV_TABLES	   2
#define UDF_VERBLEV_MAX		   3


#if 1
#	define UDF_VERBOSE(op)			if (udf_verbose) { op; }
#	define UDF_VERBOSE_LEVEL(level, op)	if (udf_verbose >= (level)) { op; }
#	define UDF_VERBOSE_TABLES(op)		UDF_VERBOSE_LEVEL(UDF_VERBLEV_TABLES, op)
#	define UDF_VERBOSE_MAX(op)		UDF_VERBOSE_LEVEL(UDF_VERBLEV_MAX, op)
#else
#	define UDF_VERBOSE(op)
#	define UDF_VERBOSE_LEVEL(level, op)
#	define UDF_VERBOSE_TABLES(op)
#	define UDF_VERBOSE_MAX(op)
#endif


#define MAX_SESSIONS		 100
#define UDF_SECTOR_SIZE		2048
#define CD_SECTOR_LENGTH	2048
#define DISC_SECTOR_LENGTH	 512

/* constants to identify what kind of identifier we are dealing with */
#define UDF_REGID_DOMAIN		 1
#define UDF_REGID_UDF			 2
#define UDF_REGID_IMPLEMENTATION	 3
#define UDF_REGID_APPLICATION		 4
#define UDF_REGID_NAME			99


#define CD_SESS_QUIRK_SESSION_LOCAL   1


typedef uint8_t udf_sector[UDF_SECTOR_SIZE];


struct udf_state {
	int	node_state;
};
#define UDF_STATE_OK		0
#define UDF_STATE_DIRTY		1
#define UDF_STATE_SHEDULED	2


struct udf_session;


/*
 * Provides :: details about a fileentry/directory/fileident
 *
 */
struct udf_icb_node {
	struct udf_state state;

	/* location; effectively long ICB ?? maybe generalise it later ? */
	struct udf_log_vol	*udf_log_vol;		/* foreign */
	struct long_ad		 location;		/* hash key ; in UDF endian for sanity   */

	union dscrptr		*dscrptr;		/* details about what is located here	 */
};


/*
 * Provides :: mountpoint -> fileset descriptor and logical volume
 *
 */
struct udf_mountpoint {
	struct udf_state state;

	char			*mount_name;		/* index too	*/
	struct udf_log_vol	*udf_log_vol;		/* foreign	*/
	struct fileset_desc	*fileset_desc;

	struct udf_mountpoint *next;			/* lists mountpoints in logical volume */
};


/*
 * Provides :: logvol_partition -> volumeset physical partition
 *
 */
struct udf_part_mapping {
	struct udf_state state;

	uint32_t		  udf_part_mapping_type;
	uint32_t		  vol_seq_num;
	uint32_t		  udf_virt_part_num;
	uint32_t		  udf_phys_part_num;
	union  udf_pmap		 *udf_pmap;		/* foreign	*/

	/* supporting tables */
	struct udf_sparing_table *sparing_table;

	struct file_entry	 *vat_file_entry;
	struct udf_vat		 *vat;
	uint8_t			 *vat_translation;
	uint32_t		  vat_entries;
	uint32_t		  vat_length;

	struct udf_icb_node	 *meta_file;
	struct udf_icb_node	 *meta_mirror_file;
	struct udf_icb_node	 *meta_bitmap_file;

	struct udf_part_mapping	 *next;
};
#define UDF_PART_MAPPING_ERROR    0
#define UDF_PART_MAPPING_PHYSICAL 1
#define UDF_PART_MAPPING_VIRTUAL  2
#define UDF_PART_MAPPING_SPARABLE 3
#define UDF_PART_MAPPING_META     4


/*
 * Provides :: log_vol -> ...
 *          :: MOUNTPOINTS :)
 */
struct udf_log_vol {
	struct udf_state state;

	int			 broken;
	
	struct udf_pri_vol 	*primary;		/* foreign */

	struct logvol_desc	*log_vol;
	struct logvol_int_desc	*logvol_integrity;

	struct hash_table	 udf_icb_nodes;
	uint32_t		 min_udf_readver;
	uint32_t		 min_udf_writever;
	uint32_t		 max_udf_writever;

	uint32_t		 num_mountpoints;	/* display only */
	struct udf_mountpoint	*mountpoints;

//	struct fileset_desc	*fileset_desc;		/* XXX needs to go */
//	uint32_t		 num_fileset_desciptors;
//	struct udf_fileset	*filesets;		/* MOUNTPOINT! */

	uint32_t		 num_part_mappings;
	struct udf_part_mapping	*part_mappings;		/* Array of part mappings */


	struct udf_log_vol 	*next;
};


/*
 * Provides :: pri_vol -> [log_vol], [part],[ ...]
 *          :: { volumeset -> [pri_vols] }
 */
struct udf_pri_vol {
	struct udf_state state;

	struct pri_vol_desc	*pri_vol;
	struct udf_session 	*udf_session;

	struct impvol_desc	*implemation;		/* most likely reduntant */
	struct udf_volumeset	*volumeset;		/* foreign ; nesisary? */
	struct unalloc_sp_desc	*unallocated;

	struct udf_log_vol 	*log_vols;		/* linked list of logical volumes in this primary */

	struct udf_pri_vol	*next;
};


/*
 * Provides :: partion -> [partition info, session]
 */
struct udf_partition {
	struct udf_state state;

	struct part_desc	*partition;
	struct udf_session	*udf_session;		/* foreign */

	struct udf_partition	*next;
};


/*
 * Provides :: volumeset -> [pri_vol]
 *          :: [volumeset]
 */
struct udf_volumeset {
	struct udf_state state;

	int			 obsolete;
	int			 max_partnum;
	struct udf_pri_vol	*primaries;		/* linked list of primaries			*/
	struct udf_partition	*parts;			/* linked list of partitions descriptors	*/

	struct udf_volumeset *next;
};


/*
 * Provides udf_session :: -> [(disc, anchor, tracknum)]
 */
struct udf_session {
	struct udf_state state;

	struct udf_discinfo	*disc;
	struct anchor_vdp	 anchor;

	uint16_t		 session_num;
	uint32_t		 session_offset;
	uint32_t		 session_length;

	struct udf_session *next;
};


/*
 * The complete disc, all cd sessions and how to access the drive
 */
struct udf_discinfo {
	struct udf_state state;

	char	*dev_name;
	int	 fhandle;

	int	 mmc_profile;			/* in case of SCSI */
	int	 devdrv_class;

	int	 recordable;			/* media is record-able; i.e. not static	*/
	int	 rewritable;			/* media can be erased selectively		*/
	int	 sector_size;			/* sector size in bytes				*/
	int	 disc_state;			/* empty - incomplete - full   - not_serial	*/
	int	 last_session_state;		/* empty - incomplete - closed - complete	*/

	int	 num_sessions;
	int	 num_udf_sessions;
	int	 session_is_UDF[MAX_SESSIONS];
	int	 session_quirks[MAX_SESSIONS];

	off_t	 session_start [MAX_SESSIONS];
	off_t	 session_end   [MAX_SESSIONS];

	uint32_t next_writable [MAX_SESSIONS];
	uint32_t free_blocks   [MAX_SESSIONS];
	uint32_t packet_size   [MAX_SESSIONS];

	struct udf_session  *sessions;
	struct udf_discinfo *next;
};

#define DISC_STATE_EMPTY		0	/* nothing on the disc	 */
#define DISC_STATE_INCOMPLETE		1	/* can be appended	 */
#define DISC_STATE_FULL			2	/* no appending possible */
#define DISC_STATE_NOT_SERIAL		3	/* not serial		 */

#define SESSION_STATE_EMPTY		0	/* still empty		 */
#define SESSION_STATE_INCOMPLETE	1	/* session is open	 */
#define SESSION_STATE_CLOSED		2	/* we need a new session */
#define SESSION_STATE_COMPLETE		3	/* no appending possible */

#define UDF_DEVDRV_CLASS		0xF00
#define UDF_DEVDRV_CLASS_FILE		0x000
#define UDF_DEVDRV_CLASS_DISC		0x100
#define UDF_DEVDRV_CLASS_CD		0x200
#define UDF_DEVDRV_CLASS_DVD		0x400
/*                                        C	- class
 *                                         R	- recordable
 *                                          W	- rewritable	*/
#define UDF_DEVDRV_TYPE_FILE		0x011
#define UDF_DEVDRV_TYPE_DISC		0x111
#define UDF_DEVDRV_TYPE_CD		0x200
#define UDF_DEVDRV_TYPE_CD_R		0x210
#define UDF_DEVDRV_TYPE_CD_RW		0x211
#define UDF_DEVDRV_TYPE_DVD		0x400
#define UDF_DEVDRV_TYPE_DVD_PLUS_R	0x410
#define UDF_DEVDRV_TYPE_DVD_PLUS_RW	0x411


struct udf_readdir_stream {
	struct udf_icb_node	*udf_icb_node;
	int		 eof;

	uint32_t	 fileno;
	uint8_t		*dirpart;
	uint32_t	 pos;
	uint64_t	 dir_len;
};


/* exported functions */
extern int  udf_check_tag(union dscrptr *dscr);
extern int  udf_check_tag_payload(union dscrptr *dscr);
extern int  udf_check_tag_presence(union dscrptr *dscr, int TAG);

extern int  udf_check_session_range(char *range);

extern int  udf_discinfo_get_type(struct udf_discinfo *disc);
extern int  udf_get_anchors(struct udf_discinfo *disc);
extern int  udf_get_volumeset_space(struct udf_discinfo *disc);
extern void udf_eliminate_predescessor_volumesets(struct udf_discinfo *disc);
extern int  udf_get_logical_volumes_info(struct udf_discinfo *disc);

extern int  udf_lookup_node(struct udf_log_vol *udf_log_vol, struct long_ad *addr, struct udf_icb_node **nodeptr);
extern int  udf_read_file_part(struct udf_icb_node *udf_icb_node, uint64_t offset, uint64_t length, void **blob, uint64_t *result_len, uint64_t *real_len);

extern int  udf_opendir(struct udf_icb_node *node, int offset, struct udf_readdir_stream **readdirptr);
extern int  udf_readdir_entry(struct udf_readdir_stream *readdir, struct dirent *entry, struct long_ad *icb, int *type);
extern int  udf_closedir(struct udf_readdir_stream *readdir);

extern int  udf_read_phys(udf_sector *result, struct udf_discinfo *disc, off_t sector, char *what);
extern int  udf_read_complete_file(struct udf_icb_node *udf_icb_node, void **blobptr, uint64_t *lengthptr);

extern int  udf_open_disc(char *dev_name, struct udf_discinfo **discptr);
extern int  udf_close_disc(struct udf_discinfo *disc);
extern int  udf_open_and_read_disc(char *devname, char *range, struct udf_discinfo **disc);

/* exported text-dump functions */
extern void udf_dump_volume_name(char *prefix, struct udf_log_vol *udf_log_vol);
extern void udf_dump_long_ad(char *prefix, struct long_ad *adr);
extern void udf_dump_id(char *prefix, int len, char *id, struct charspec *chsp);
extern void udf_to_unix_name(char *result, char *id, int len, struct charspec *chsp);


/* statics inside udf */
extern struct udf_discinfo  *udf_discs_list;
extern struct udf_volumeset *udf_volumeset_list;
extern struct hash_table    *udf_mountables;


#endif	/* _UDF_H_ */

